from __future__ import annotations

import os
import sys

from .section import IniConfigSection

from configparser import ConfigParser
from typing import Any, List, Optional, Union


class IniKlassException(Exception):
    pass


class IniConfig:
    config: IniConfigSection

    @staticmethod
    def read(file=None, empty_to_none=False) -> IniConfig:
        """
        Load configuration by given path, command line argument or
        `CONFIG` environment.

        :param Optional[str] file: the configuration file path
        :return: the configuration object
        :rtype: ini.IniConfig
        """
        c = ConfigParser(default_section=None)

        path = None
        if file is not None:
            path = file
        elif 'CONFIG' in os.environ:
            path = os.environ.get('CONFIG')
        elif len(sys.argv) > 1:
            path = sys.argv[1]
        else:
            raise IniKlassException('Configuration parameter must be set.')

        if path is None or len(path.strip()) == 0:
            raise IniKlassException('Configuration path is missing.')

        if not os.path.exists(path):
            raise IniKlassException(f'Configuration file does not exists "{path}".')

        # read configuration from local path
        c.read(path)

        # size of sections must be greater than one
        if len(c.sections()) == 0:
            raise IniKlassException('There are no any sections.')

        ini = IniConfig()
        ini.config = IniConfigSection(c, empty_to_none)

        return ini

    def __getattr__(self, key: str) -> Optional[Union[str, float, bool, List[Any]]]:
        """
        Get configuration parameter

        :param str key: class parameter of configuration
        :return: configuration value
        :rtype: Optional[Union[str, float, bool, List[Any]]]
        """
        if not hasattr(self.config, key):
            return None

        return getattr(self.config, key)

    def __repr__(self) -> str:
        """
        Represent class content

        :return: content as text
        :rtype: str
        """
        kv = dict()

        for k in vars(self.config):
            kv[k] = getattr(self, k)

        return str(kv)



from ini import IniConfig
from pathlib import Path
from unittest import TestCase


class SimpleTest(TestCase):
    def setUp(self):
        path = Path(__file__).parent / 'simple.ini'

        self.config = IniConfig.read(str(path))

    def test_numeric(self):
        self.assertEqual(self.config.example.id, 12)

    def test_string(self):
        self.assertEqual(self.config.example.name, 'John')

    def test_boolean(self):
        self.assertEqual(self.config.example.status, True)

    def test_float(self):
        self.assertEqual(self.config.example.salary, 100.5)

    def test_list(self):
        self.assertEqual(self.config.example.group, ['car', 'book', 'phone'])



from ini import IniConfig
from pathlib import Path
from unittest import TestCase


class NullableTest(TestCase):
    def setUp(self):
        path = Path(__file__).parent / 'simple.ini'

        self.config = IniConfig.read(str(path), empty_to_none=True)

    def test_null_array(self):
        self.assertEqual(self.config.nulls.age, None)

    def test_string(self):
        self.assertEqual(self.config.nulls.users, [1, 2, None])



from .item import IniConfigItem
from configparser import SectionProxy
from dataclasses import dataclass
from typing import Optional


@dataclass(unsafe_hash=True)
class IniConfigSection:
    def __init__(
        self,
        item: SectionProxy,
        empty_to_none: Optional[bool] = False,
    ) -> None:
        """
        Generate config item from given config parser section

        :param SectionProxy item: config parser section
        :param Optional[bool] empty_to_none: use None for empty values. Default is false
        """
        for k, v in item.items():
            if k is None:
                continue

            setattr(self, k, IniConfigItem(v, empty_to_none))

    def __repr__(self) -> str:
        """
        Represent class content

        :return: content as text
        :rtype: str
        """
        kv = dict()

        for k in vars(self):
            kv[k] = getattr(self, k)

        return str(kv)



from setuptools import find_packages, setup

description = 'ini file parser.'

try:
    long_description = open('readme.md', 'r', encoding='utf8').read()
except Exception:
    long_description = description

setup(
    name='ini-klass',
    version='0.0.3',
    author='Tarik Yilmaz',
    author_email='tarikyilmaz.54@gmail.com',
    description=description,
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/trk54ylmz/ini-klass',
    include_package_data=True,
    packages=find_packages(),
    python_requires='>=3.7',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Operating System :: MacOS',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ],
)
